-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- This file provide a simple row based grid layout widget.  It can be used to simplify the display of tablular data
--
--

--
-- Global variables
--
local addon, Attend = ...
Attend.RowLayout = {}
Attend.RowLayout.debug = true
Attend.RowLayout.name = "Attend.RowLayout"
Attend.RowLayout.resizeFrame = 1
Attend.RowLayout.centerFrame = 2

--
-- Main functions
--

-- Attend.RowLayout.Create(frame, rows, cols, rowHeight, colWidth, rowMargin, colMargin)
-- Params:
--   frame     - the frame for the grid layout to use
--   rows      - the number of rows in the grid layout
--   cols      - the number of columns in the grid layout
--   rowHeight - the default height to make all the grid rows
--   colWidth  - the default width to make all columns
--   rowMargin - the gap to leave between rows
--   colMargin - the gap to leave between cols
-- Returns: the handle to the grid layout control
function Attend.RowLayout.Create(frame, rows, cols, rowHeight, colWidth, rowMargin, colMargin)
	if (rows < 1) then
		print(string.format("%s: cannot create a grid with %d rows", Attend.RowLayout.name, rows))
		return nil
	end
	if (cols < 1) then
		print(string.format("%s: cannot create a grid with %d cols", Attend.RowLayout.name, cols))
		return nil
	end
	if (rowHeight < 1) then
		print(string.format("%s: cannot create a grid with %d row height", Attend.RowLayout.name, rowHeight))
		return nil
	end
	if (colWidth < 1) then
		print(string.format("%s: cannot create a grid with %d col width", Attend.RowLayout.name, colWidth))
		return nil
	end
	if (rowMargin < 0) then
		print(string.format("%s: cannot create a grid with %d row margin", Attend.RowLayout.name, rowMargin))
		return nil
	end
	if (colMargin < 0) then
		print(string.format("%s: cannot create a grid with %d col margin", Attend.RowLayout.name, colMargin))
		return nil
	end
	
	local rowLayout = {}
	rowLayout.rowCount = rows
	rowLayout.colCount = cols
	rowLayout.rowMargin = rowMargin
	rowLayout.colMargin = colMargin
	rowLayout.frame = frame

	rowLayout.rows = {}
	for indx = 1, rows do
		local colArray = {}
		for indx2 = 1, cols do
			local colEntry = {}
			colEntry.frame = nil
			colEntry.mask = nil
			table.insert(colArray, colEntry)
		end
		
		local thisRow = {}
		thisRow.frame = UI.CreateFrame("Frame", string.format("Attend.RowLayoutRow%d", indx), rowLayout.frame)
		thisRow.height = rowHeight
		thisRow.cols = colArray
		thisRow.visible = true
		table.insert(rowLayout.rows, thisRow)
	end
	
	rowLayout.colWidths = {}
	for indx = 1, cols do
		table.insert(rowLayout.colWidths, colWidth);
	end
	
	
	return rowLayout
end

-- Attend.RowLayout.CreateFrame(rowLayout, row, col, frameType)
--   Creates a UI frame for the particular row,col in the grid
-- Params:
--   rowLayout     - the row layout handle
--   row           - the number of rows in the grid layout
--   col           - the number of columns in the grid layout
--   frameType     - the y position of the grid layout frame
--   framePosition - Attend.RowLayout.resizeFrame (or nil) to have the frame sized to fit the space, Attend.RowLayout.centerFrame to position it into the cell's center
-- Returns: the handle to the created frame
function Attend.RowLayout.CreateFrame(rowLayout, row, col, frameType, framePosition)
	if not rowLayout then
		print(string.format("%s: cannot CreateFrame on a nil layout", Attend.RowLayout.name))
		return
	end
	if (row < 1 or row > rowLayout.rowCount) then
		print(string.format("%s: cannot create a frame on row %d inside a layout with %d rows", Attend.RowLayout.name, row, rowLayout.rowCount))
		return nil
	end
	if (col < 1 or col > rowLayout.colCount) then
		print(string.format("%s: cannot create a frame on col %d inside a layout with %d cols", Attend.RowLayout.name, col, rowLayout.colCount))
		return nil
	end
	
	rowLayout.rows[row].cols[col].mask = UI.CreateFrame("Mask", string.format("Attend.RowLayoutCellMask%d-%d", row, col), rowLayout.rows[row].frame)
	rowLayout.rows[row].cols[col].frame = UI.CreateFrame(frameType, string.format("Attend.RowLayoutCell%d-%d", row, col), rowLayout.rows[row].cols[col].mask)
	rowLayout.rows[row].cols[col].mask:SetAllPoints(rowLayout.rows[row].cols[col].frame)
	if framePosition then
		rowLayout.rows[row].cols[col].framePosition = framePosition
	else
		rowLayout.rows[row].cols[col].framePosition = Attend.RowLayout.resizeFrame
	end
	
	return rowLayout.rows[row].cols[col].frame
end

-- Attend.RowLayout.SetRowHeight(rowLayout, row, height)
--   Changes the height of a whole row in the grid
-- Params:
--   rowLayout - the row layout handle
--   row       - the row number to change
--   height    - the new height of the given row
-- Returns: nothing
function Attend.RowLayout.SetRowHeight(rowLayout, row, height)
	if not rowLayout then
		print(string.format("%s: cannot SetRowHeight on a nil layout", Attend.RowLayout.name))
		return
	end
	if (row < 1 or row > rowLayout.rowCount) then
		print(string.format("%s: row %d is not inside a layout with %d rows", Attend.RowLayout.name, row, rowLayout.rowCount))
		return
	end
	if (height < 1) then
		print(string.format("%s: height %d must be greater than zero", Attend.RowLayout.name, height))
		return
	end
	
	rowLayout.rows[row].height = height
end

-- Attend.RowLayout.SetColWidth(rowLayout, col, width)
--   Changes the width of a whole column in the grid
-- Params:
--   rowLayout - the row layout handle
--   col       - the column number to change
--   width    - the new width of the given col
-- Returns: nothing
function Attend.RowLayout.SetColWidth(rowLayout, col, width)
	if not rowLayout then
		print(string.format("%s: cannot SetColWidth on a nil layout", Attend.RowLayout.name))
		return
	end
	if (col < 1 or col > rowLayout.colCount) then
		print(string.format("%s: col %d is not inside a layout with %d cols", Attend.RowLayout.name, col, rowLayout.colCount))
		return
	end
	if (width < 1) then
		print(string.format("%s: width %d must be greater than zero", Attend.RowLayout.name, width))
		return
	end
	
	rowLayout.colWidths[col] = width
end

-- Attend.RowLayout.SetRowVisible(rowLayout, row, visible)
--   Changes the visibility of the whole row
-- Params:
--   rowLayout - the row layout handle
--   row       - the row number to change
--   visible   - the new visibility of the given row
-- Returns: nothing
function Attend.RowLayout.SetRowVisible(rowLayout, row, visible)
	if not rowLayout then
		print(string.format("%s: cannot SetRowVisible on a nil layout", Attend.RowLayout.name))
		return
	end
	if (row < 1 or row > rowLayout.rowCount) then
		print(string.format("%s: row %d is not inside a layout with %d rows", Attend.RowLayout.name, row, rowLayout.rowCount))
		return
	end
	
	rowLayout.rows[row].visible = visible
end

-- Attend.RowLayout.Redraw(rowLayout)
--   This function causes the layout to recalculate all dimensions and refresh on the screen.  It must be called after changes have been made to the layout
-- Params:
--   rowLayout - the rowLayout frame to refresh
-- Returns: none
function Attend.RowLayout.Redraw(rowLayout)
	if not rowLayout then
		print(string.format("%s: cannot redraw a nil layout", Attend.RowLayout.name))
		return
	end

	-- calculate the total width
	local totalWidth = 0
	for colNumber = 1, rowLayout.colCount do
		totalWidth = totalWidth + rowLayout.colWidths[colNumber] + rowLayout.colMargin
	end
	totalWidth = totalWidth - rowLayout.colMargin

	-- calculate the total height
	local totalHeight = 0
	for rowNumber = 1, rowLayout.rowCount do
		if rowLayout.rows[rowNumber].visible == true then
			totalHeight = totalHeight + rowLayout.rows[rowNumber].height + rowLayout.rowMargin
		end
	end
	if totalHeight >= rowLayout.rowMargin then
		totalHeight = totalHeight - rowLayout.rowMargin
	end
	
	rowLayout.frame:SetWidth(totalWidth);
	rowLayout.frame:SetHeight(totalHeight);
	
	-- now redraw each row moving each of its column cells in the process
	local currentY = 0
	for rowNumber = 1, rowLayout.rowCount do
		rowLayout.rows[rowNumber].frame:SetVisible(rowLayout.rows[rowNumber].visible)
		rowLayout.rows[rowNumber].frame:SetPoint("TOPLEFT", rowLayout.frame, "TOPLEFT", 0, currentY)
		rowLayout.rows[rowNumber].frame:SetHeight(rowLayout.rows[rowNumber].height)
		rowLayout.rows[rowNumber].frame:SetWidth(totalWidth)

		local currentX = 0
		for colNumber = 1, rowLayout.colCount do
			if rowLayout.rows[rowNumber].cols[colNumber].frame then
				local resize = false		
				if rowLayout.rows[rowNumber].cols[colNumber].framePosition == Attend.RowLayout.resizeFrame then
					resize = true
				else
					if rowLayout.rows[rowNumber].cols[colNumber].frame:GetHeight() > rowLayout.rows[rowNumber].height or
					   rowLayout.rows[rowNumber].cols[colNumber].frame:GetWidth() > rowLayout.colWidths[colNumber] then
						resize = true
					else
						local widthAdj = math.floor(rowLayout.colWidths[colNumber] / 2 - rowLayout.rows[rowNumber].cols[colNumber].frame:GetWidth() / 2)
						local heightAdj = math.floor(rowLayout.rows[rowNumber].height / 2 - rowLayout.rows[rowNumber].cols[colNumber].frame:GetHeight() / 2)
						rowLayout.rows[rowNumber].cols[colNumber].frame:SetPoint("TOPLEFT", rowLayout.rows[rowNumber].frame, "TOPLEFT", currentX + widthAdj, heightAdj)
					end
				end

				if resize == true then
					rowLayout.rows[rowNumber].cols[colNumber].frame:SetPoint("TOPLEFT", rowLayout.rows[rowNumber].frame, "TOPLEFT", currentX, 0)
					rowLayout.rows[rowNumber].cols[colNumber].frame:SetHeight(rowLayout.rows[rowNumber].height)
					rowLayout.rows[rowNumber].cols[colNumber].frame:SetWidth(rowLayout.colWidths[colNumber])
				end
			end
			
			currentX = currentX + rowLayout.colWidths[colNumber] + rowLayout.colMargin
		end
		
		if rowLayout.rows[rowNumber].visible == true then
			currentY = currentY + rowLayout.rows[rowNumber].height + rowLayout.rowMargin
		end
	end
end

-- Attend.RowLayout.GetFrame(rowLayout)
--   Get the top level frame for the whole row layout grid
-- Params:
--   rowLayout - the row layout handle
-- Returns: the top level frame
function Attend.RowLayout.GetFrame(rowLayout)
	if not rowLayout then
		print(string.format("%s: cannot GetFrame on a nil layout", Attend.RowLayout.name))
		return
	end
	
	return rowLayout.frame
end


-- Attend.RowLayout.GetCellFrame(rowLayout, row, col)
--   Retrieves a UI frame for the particular row,col in the grid
-- Params:
--   rowLayout - the row layout handle
--   row       - the number of rows in the grid layout
--   col       - the number of columns in the grid layout
-- Returns: the handle to the created frame
function Attend.RowLayout.GetCellFrame(rowLayout, row, col)
	if not rowLayout then
		print(string.format("%s: cannot GetCellFrame on a nil layout", Attend.RowLayout.name))
		return
	end
	if (row < 1 or row > rowLayout.rowCount) then
		print(string.format("%s: cannot create a frame on row %d inside a layout with %d rows", Attend.RowLayout.name, row, rowLayout.rowCount))
		return nil
	end
	if (col < 1 or col > rowLayout.colCount) then
		print(string.format("%s: cannot create a frame on col %d inside a layout with %d cols", Attend.RowLayout.name, col, rowLayout.colCount))
		return nil
	end
	
	return rowLayout.rows[row].cols[col].frame
end

-- Attend.RowLayout.GetNumRows(rowLayout)
--   Get the number of rows in the layout grid
-- Params:
--   rowLayout - the row layout handle
-- Returns: the number of rows
function Attend.RowLayout.GetNumRows(rowLayout)
	if not rowLayout then
		print(string.format("%s: cannot GetNumRows on a nil layout", Attend.RowLayout.name))
		return
	end
	
	return rowLayout.rowCount
end


-- Attend.RowLayout.GetNumCols(rowLayout)
--   Get the number of columns in the layout grid
-- Params:
--   rowLayout - the row layout handle
-- Returns: the number of columns
function Attend.RowLayout.GetNumCols(rowLayout)
	if not rowLayout then
		print(string.format("%s: cannot GetNumCols on a nil layout", Attend.RowLayout.name))
		return
	end
	
	return rowLayout.colCount
end


-- Attend.RowLayout.GetRowHeight(rowLayout, row)
--   Get the height of the given row
-- Params:
--   rowLayout - the row layout handle
--   row       - the row in the grid layout to query
-- Returns: the row height
function Attend.RowLayout.GetRowHeight(rowLayout, row)
	if not rowLayout then
		print(string.format("%s: cannot GetRowHeight on a nil layout", Attend.RowLayout.name))
		return
	end
	if (row < 1 or row > rowLayout.rowCount) then
		print(string.format("%s: row %d out of range 1-%d", Attend.RowLayout.name, row, rowLayout.rowCount))
		return nil
	end
	
	return rowLayout.rows[row].height
end


-- Attend.RowLayout.GetColWidth(rowLayout, col)
--   Get the width of the given column
-- Params:
--   rowLayout - the row layout handle
--   col       - the column in the grid layout to query
-- Returns: the column height
function Attend.RowLayout.GetColWidth(rowLayout, col)
	if not rowLayout then
		print(string.format("%s: cannot GetColWidth on a nil layout", Attend.RowLayout.name))
		return
	end
	if (col < 1 or col > rowLayout.colCount) then
		print(string.format("%s: cannot col %d out of range 1-%d", Attend.RowLayout.name, col, rowLayout.colCount))
		return nil
	end
	
	return rowLayout.colWidths[col]
end


