-- Based on original code written by Gordon Tyler

-- Helper Functions

local function contains(tbl, val)
  for k, v in pairs(tbl) do
    if v == val then
      return true
    end
  end
  return false
end

local function UpdateSelection(self, index)
  local item = self.items[index]

  if index == nil then
    self.current:SetText("Select...")
	self.current:SetFontColor(0.5, 0.5, 0.5, 0.5)
  else
    self.current:SetText(item)
	self.current:SetFontColor(1, 1, 1, 1)
  end
end

local function GetTextureName(widget)
	local name = "textures/DownButton.png"
	if widget.enabled then
		local mouse = Inspect.Mouse()
		local left, top, right, bottom = widget.current:GetBounds()
		if mouse.x >= left and mouse.x <= right and mouse.y >= top and mouse.y < bottom then
			name = "textures/DownButtonMouseOver.png"
		end

		local left, top, right, bottom = widget.button:GetBounds()
		if mouse.x >= left and mouse.x <= right and mouse.y >= top and mouse.y < bottom then
			name = "textures/DownButtonMouseOver.png"
		end
	end
	
	return name
end


-- Current Frame Events

local function CurrentClick(self)
  local widget = self:GetParent()
  if not widget.enabled then return end
  local dropdown = widget.dropdown
  dropdown:SetVisible(not dropdown:GetVisible())
end

local function RedrawButton(self)
  local widget = self:GetParent()
  widget.button:SetTexture("LibSimpleWidgetsExt", GetTextureName(widget))
  if widget.enabled then
	widget.button:SetAlpha(1)
  else
	widget.button:SetAlpha(0.5)
  end
end

-- Dropdown Frame Events

local function DropdownItemClick(self)
  local widget = self:GetParent():GetParent():GetParent()
  local item = self:GetText()
  widget.current:SetText(item)
  widget.dropdown:SetVisible(false)
  widget:SetSelectedIndex(self.index)
end

local function DropdownItemMouseIn(self)
  self:SetBackgroundColor(0.3, 0.3, 0.3, 1)
end

local function DropdownItemMouseOut(self)
  self:SetBackgroundColor(0, 0, 0, 0)
end


-- Public Functions

local function SetBorder(self, width, r, g, b, a)
  Library.LibSimpleWidgets.SetBorder(self.current, width, r, g, b, a)
  Library.LibSimpleWidgets.SetBorder(self.dropdown, width, r, g, b, a)
end

local function SetBackgroundColor(self, r, g, b, a)
  self.current:SetBackgroundColor(r, g, b, a)
  self.dropdown:SetBackgroundColor(r, g, b, a)
end

local function GetFontSize(self)
  return self.current:GetFontSize()
end

local function SetFontSize(self, size)
  self.current:SetFontSize(size)
  self:SetHeight(self.current:GetHeight())
  local height = 0
  for i, itemFrame in ipairs(self.itemFrames) do
    itemFrame:SetFontSize(size)
    itemFrame:SetHeight(itemFrame:GetHeight())
    height = height + itemFrame:GetHeight()
  end
end

local function GetDefaultHeight(self)
  return self.current:GetHeight()
end

local function GetDefaultWidth(self)
  local maxWidth = self.current:GetWidth()
  for i, itemFrame in ipairs(self.itemFrames) do
    if itemFrame:GetVisible() then
      maxWidth = math.max(maxWidth, itemFrame:GetWidth())
    end
  end
  return maxWidth
end

local function ResizeToDefault(self)
  self:SetWidth(GetDefaultWidth())
  self:SetHeight(GetDefaultHeight())
end

local function GetEnabled(self)
  return self.enabled
end

local function SetEnabled(self, enabled)
  self.enabled = enabled
  if enabled then
	UpdateSelection(self, self.selectedIndex)
  else
    self.current:SetFontColor(0.5, 0.5, 0.5, 1)
    self.dropdown:SetVisible(false)
  end
  
  RedrawButton(self.button)
end

local function SetHeight(self, height)
	self:OldSetHeight(height)
	self.button:SetWidth(height)
end

local function GetItems(self)
  return self.items
end

local function SetItems(self, items, values)
  self.items = items
  self.values = values or {}

  local dropdownContent = UI.CreateFrame("Frame", self.dropdown:GetName().."DropdownContent", self.dropdown)
  self.itemFrames = {}

  -- setup item frames
  local dropdownHeight = 0
  local prevItemFrame
  for i, v in ipairs(items) do
    local itemFrame
    if not self.itemFrames[i] then
      itemFrame = UI.CreateFrame("Text", self.dropdown:GetName().."Item"..i, dropdownContent)
      if prevItemFrame then
        itemFrame:SetPoint("TOPLEFT", prevItemFrame, "BOTTOMLEFT")
        itemFrame:SetPoint("TOPRIGHT", prevItemFrame, "BOTTOMRIGHT")
      else
        itemFrame:SetPoint("TOPLEFT", dropdownContent, "TOPLEFT")
        itemFrame:SetPoint("TOPRIGHT", dropdownContent, "TOPRIGHT")
      end
      itemFrame.Event.LeftClick = DropdownItemClick
      itemFrame.Event.MouseIn = DropdownItemMouseIn
      itemFrame.Event.MouseOut = DropdownItemMouseOut
      itemFrame.index = i
      self.itemFrames[i] = itemFrame
    else
      itemFrame = self.itemFrames[i]
    end
    itemFrame:SetText(v)
    itemFrame:SetHeight(itemFrame:GetHeight())
    itemFrame:SetVisible(true)
    dropdownHeight = dropdownHeight + itemFrame:GetHeight()
    prevItemFrame = itemFrame
  end

  if #items > 0 then
	if #items < self.rows then
		self.dropdown:SetHeight(19 * #items)
	else
		self.dropdown:SetHeight(19 * self.rows)
	end
  end
  
  dropdownContent:SetHeight(dropdownHeight)
  self.dropdown:SetContent(dropdownContent)

  self.selectedIndex = nil
  UpdateSelection(self, nil)
end

local function GetValues(self)
  return self.values
end

local function GetSelectedItem(self)
  return self.items[self.selectedIndex]
end

local function SetSelectedItem(self, item)
  if item then
    for i, v in ipairs(self.items) do
      if v == item then
        self:SetSelectedIndex(i)
        return
      end
    end
  end

  self:SetSelectedIndex(nil)
end

local function GetSelectedValue(self)
  return self.values[self.selectedIndex]
end

local function SetSelectedValue(self, value)
  if value then
    for i, v in ipairs(self.values) do
      if v == value then
        self:SetSelectedIndex(i)
        return
      end
    end
  end

  self:SetSelectedIndex(nil)
end

local function GetSelectedIndex(self)
  return self.selectedIndex
end

local function SetSelectedIndex(self, index)
  if index and (index < 1 or index > #self.items) then
    index = nil
  end

  if index == self.selectedIndex then
    return
  end

  self.selectedIndex = index
  UpdateSelection(self, index)

  if self.Event.ItemSelect then
    local item = self.items[index]
    local value = self.values[index]
    self.Event.ItemSelect(self, item, value, index)
  end
end


-- Constructor Function

function Library.LibSimpleWidgetsExt.Select(name, parent)
  local widget = UI.CreateFrame("Frame", name, parent)

  widget.enabled = true
  widget.items = {}
  widget.values = {}
  widget.itemFrames = {}
  widget.selectedIndex = nil
  widget.rows = 10

  widget.button = UI.CreateFrame("Texture", widget:GetName().."Button", widget)
  widget.button:SetPoint("TOPRIGHT", widget, "TOPRIGHT")
  widget.button:SetPoint("BOTTOMRIGHT", widget, "BOTTOMRIGHT")
  widget.button:SetTexture("LibSimpleWidgetsExt", "textures/DownButton.png")
  widget.button.Event.LeftClick = CurrentClick
  widget.button.Event.MouseIn = RedrawButton
  widget.button.Event.MouseOut = RedrawButton
  
  widget.current = UI.CreateFrame("Text", widget:GetName().."Current", widget)
  widget.current:SetPoint("TOPLEFT", widget, "TOPLEFT")
  widget.current:SetPoint("BOTTOMRIGHT", widget.button, "BOTTOMLEFT", -1, 0)
  widget.current:SetBackgroundColor(0, 0, 0, 1)
  widget.current:SetText("Select...")
  widget.current.Event.LeftClick = CurrentClick  
  widget.current.Event.MouseIn = RedrawButton
  widget.current.Event.MouseOut = RedrawButton

  local scrollInterval = widget.current:GetHeight()
  widget.dropdown = UI.CreateFrame("SimpleScrollView", widget:GetName().."Dropdown", widget)
  widget.dropdown:SetLayer(widget:GetLayer() + 5)
  widget.dropdown:SetBackgroundColor(0, 0, 0, 1)
  widget.dropdown:SetPoint("TOPLEFT", widget, "BOTTOMLEFT", 0, 5)
  widget.dropdown:SetPoint("TOPRIGHT", widget, "BOTTOMRIGHT", 0, 5)
  widget.dropdown:SetHeight(scrollInterval * widget.rows)
  widget.dropdown:SetScrollInterval(scrollInterval)
  widget.dropdown:SetVisible(false)

  widget.SetBorder = SetBorder
  widget.SetBackgroundColor = SetBackgroundColor
  widget.GetFontSize = GetFontSize
  widget.SetFontSize = SetFontSize
  widget.GetDefaultHeight = GetDefaultHeight
  widget.GetDefaultWidth = GetDefaultWidth
  widget.ResizeToDefault = ResizeToDefault
  widget.GetEnabled = GetEnabled
  widget.SetEnabled = SetEnabled
  widget.OldSetHeight = widget.SetHeight
  widget.SetHeight = SetHeight
  widget.GetItems = GetItems
  widget.SetItems = SetItems
  widget.GetValues = GetValues
  widget.GetSelectedIndex = GetSelectedIndex
  widget.SetSelectedIndex = SetSelectedIndex
  widget.GetSelectedItem = GetSelectedItem
  widget.SetSelectedItem = SetSelectedItem
  widget.GetSelectedValue = GetSelectedValue
  widget.SetSelectedValue = SetSelectedValue

  Library.LibSimpleWidgets.EventProxy(widget, {"ItemSelect"})

  widget.button:SetWidth(widget:GetDefaultHeight())

  return widget
end
